#
# a simple class to store mountains
#
# attributes: name
#             elevation
#             prominence
#             latitude (tuple)
#             longitude (tuple)
#             climbed
#
# author: David Mathias
#

class Mountain():
    # class variable
    num_mountains = 0
    
    def __init__(self, name, elev, prom, lat, lon):
        self.name = name
        self.elevation = elev
        self.prominence = prom
        self.latitude = lat
        self.longitude = lon
        self.note = ''
        self.climbed = False
        Mountain.num_mountains += 1
        
    def __del__(self):
        Mountain.num_mountains -= 1
    
    # convert the object to a string and return it
    # invoked by casting the object to str
    def __str__(self):
        lat = self.latitude
        lon = self.longitude
        s = 'Name: ' + self.name + '\n'
        s += 'Elevation: ' + str(self.elevation) + '\n'
        s += 'Prominence: ' + str(self.prominence) + '\n'
        s += 'Latitude: ' + lat[0] + ':' + lat[1] + ':' + lat[2] + '\n'
        s += 'Longitude: ' + lon[0] + ':' + lon[1] + ':' + lon[2] + '\n'
        s += 'Climbed: ' + str(self.climbed) + '\n'
        if self.note != '':
            s += 'Note: ' + self.note + '\n'
        return s
        
    def __repr__(self):
        return f'Mountain(name={self.name!r}, elev={self.elevation}, prom={self.prominence}, lat={self.latitude}, lon={self.longitude})'
    
    def __gt__(self, m2):
        return ((self.elevation > m2.elevation) or (self.elevation == m2.elevation and self.prominence > m2.prominence))

    def __lt__(self, m2):
        return ((self.elevation < m2.elevation) or (self.elevation == m2.elevation and self.prominence < m2.prominence))

    def __iadd__(self, gain):
        self.elevation += gain
        return self
    
    def __isub__(self, loss):
        self.elevation -= loss
        return self
    
    def __bool__(self):
        return self.elevation >= 2000
    
    def is_higher(self, m2):
        return ((self.elevation > m2.elevation) or (self.elevation == m2.elevation and self.prominence > m2.prominence))
    
    def climb(self):
        self.climbed = True
    
    def is_climbed(self):
        return self.climbed
    
    def __call__(self, s):
        self.note = s
        
